package com.hfkj.service.order;

import com.hfkj.common.exception.BaseException;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.entity.*;
import com.hfkj.model.GasPayPriceModel;
import com.hfkj.model.order.OrderChildModel;
import com.hfkj.service.agent.BsAgentMerAccountService;
import com.hfkj.service.merchant.BsMerchantService;
import com.hfkj.service.agent.BsAgentMerService;
import com.hfkj.service.gas.BsGasClassGroupTaskService;
import com.hfkj.service.gas.BsGasOrderService;
import com.hfkj.service.gas.BsGasService;
import com.hfkj.service.gas.BsGasStaffService;
import com.hfkj.service.sec.SecDictionaryService;
import com.hfkj.sysenum.merchant.MerchantSourceTypeEnum;
import com.hfkj.sysenum.merchant.MerchantStatusEnum;
import com.hfkj.sysenum.gas.GasOrderReceiptStatusEnum;
import com.hfkj.sysenum.gas.OrderOilStatus;
import com.hfkj.sysenum.merchant.MerchantAccountRecordSourceTypeEnum;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

/**
 * 创建订单业务
 * @className: OrderCreateService
 * @author: HuRui
 * @date: 2024/6/7
 **/
@Component
public class OrderCreateService {
    Logger log = LoggerFactory.getLogger(OrderCreateService.class);

    @Resource
    private BsMerchantService merchantService;
    @Resource
    private SecDictionaryService secDictionaryService;
    @Resource
    private BsGasService gasService;
    @Resource
    private BsGasOrderService gasOrderService;
    @Resource
    private BsAgentMerAccountService agentMerAccountService;
    @Resource
    private BsGasClassGroupTaskService gasClassGroupTaskService;
    @Resource
    private BsAgentMerService agentMerService;
    @Resource
    private BsGasStaffService gasStaffService;

    /**
     * 加油业务
     * @param order  订单
     * @param orderDeduction 优惠
     * @param orderChild   商品订单
     * @return
     */
    public OrderChildModel oilHandle(BsOrder order, BsOrderDeduction orderDeduction, OrderChildModel orderChild) {
        // 查询油品类型
        SecDictionary gasOilType = secDictionaryService.getDictionary("OIL_NO", orderChild.getGasOilNo());
        if (gasOilType == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知油品类型");
        }
        // 汽油限制加油金额
        if ("1".equals(gasOilType.getExt1())) {
            SecDictionary priceLimit = secDictionaryService.getDictionary("GAS_REFUEL_PRICE_LIMIT", "1");
            if (priceLimit!=null) {
                if (orderChild.getProductPrice().compareTo(new BigDecimal(priceLimit.getCodeName())) > 0) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "加油单笔限制" + priceLimit.getCodeName()+"元");
                }
            }
        }
        // 查询油站
        BsMerchant merchant = merchantService.getMerchant(orderChild.getMerNo());
        if (merchant == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的油站");
        }
        if (!merchant.getStatus().equals(MerchantStatusEnum.status1.getNumber())) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "当前油站暂停交易");
        }
        BsGasOrder gasOrder = new BsGasOrder();
        gasOrder.setAbnormal(false);
        gasOrder.setMerNo(merchant.getMerNo());
        gasOrder.setChannelType(merchant.getSourceType());
        gasOrder.setOrderNo(orderChild.getOrderNo());
        gasOrder.setOrderChildNo(orderChild.getChildOrderNo());
        gasOrder.setUserId(order.getUserId());
        gasOrder.setUserPhone(order.getUserPhone());
        gasOrder.setMerChainBrandId(merchant.getChainBrandId());
        gasOrder.setMerChainBrandName(merchant.getChainBrandName());
        gasOrder.setMerId(merchant.getId());
        gasOrder.setMerName(merchant.getMerName());
        gasOrder.setMerAddress(merchant.getAddress());
        gasOrder.setStatus(OrderOilStatus.STATUS1.getNumber());
        gasOrder.setReceiptStatus(GasOrderReceiptStatusEnum.status0.getStatus());

        // 代理、业务员
        if (orderChild.getAgentMerId() != null) {
            // 查询代理
            BsAgentMer agentMer = agentMerService.getDetail(orderChild.getAgentMerId());
            if (agentMer == null || agentMer.getStatus() != 1) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "无法交易，代理商二维码已失效");
            }
            try {
                if (MerchantSourceTypeEnum.type1.getNumber().equals(merchant.getSourceType())) {
                    // 扣除代理商商户金额
                    Map<String,Object> param = new HashMap<>();
                    param.put("sourceType", MerchantAccountRecordSourceTypeEnum.type2.getType());
                    param.put("sourceOrderNo", gasOrder.getOrderNo());
                    param.put("sourceOrderContent", "创建加油订单："+gasOrder.getOrderNo()+"，加油金额：￥" +orderChild.getProductPrice());
                    agentMerAccountService.consume(agentMer.getAgentId(), agentMer.getMerNo(), orderChild.getProductPrice(), param);
                }
            } catch (BaseException e) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, e.getErrorMsg());
            } catch (Exception e) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "扣款失败，请稍后重新提交订单");
            }
            gasOrder.setAgentId(agentMer.getAgentId());
            gasOrder.setAgentName(agentMer.getAgentName());
            gasOrder.setAgentStaffId(agentMer.getAgentStaffId());
            gasOrder.setAgentStaffName(agentMer.getAgentStaffName());
        }

        // 计算价格
        GasPayPriceModel priceModel = gasService.refuelPriceCompute(orderChild.getProductPrice(), merchant.getMerNo(), orderChild.getGasOilNo(), orderChild.getAgentMerId());

        gasOrder.setDeductionCouponPrice(new BigDecimal("0"));
        gasOrder.setDeductionOilPrice(priceModel.getTotalPreferences());
        gasOrder.setPayablePrice(priceModel.getPayPrice());
        gasOrder.setTotalDeductionPrice(gasOrder.getDeductionOilPrice().add(gasOrder.getDeductionCouponPrice()));

        // 油价信息
        gasOrder.setGasRefuelPrice(orderChild.getProductPrice());
        gasOrder.setGasOilNo(orderChild.getGasOilNo());
        gasOrder.setGasGunNo(orderChild.getGasGunNo());
        gasOrder.setGasOilType(Integer.parseInt(gasOilType.getExt1()));
        gasOrder.setGasPriceGun(priceModel.getPriceGun());
        gasOrder.setGasPriceVip(priceModel.getPriceVip());
        gasOrder.setGasPriceCost(priceModel.getPriceCost());
        gasOrder.setGasPriceOfficial(priceModel.getPriceOfficial());
        gasOrder.setGasPricePlatform(priceModel.getPricePlatform());
        gasOrder.setGasOilLiters(priceModel.getOilLiters());
        gasOrder.setGasDiscount(priceModel.getDiscount());
        gasOrder.setGasOilSubsidy(priceModel.getPreferentialMargin());
        gasOrder.setGasLitersPreferences(priceModel.getLitersPreferences());
        gasOrder.setGasPricePreferences(priceModel.getPricePreferences());

        // 加油员
        if (orderChild.getGasStaffId() != null) {
            // 查询员工
            BsGasStaff gasStaff = gasStaffService.getStaffDetailById(orderChild.getGasStaffId());
            if (gasStaff == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的加油员");
            }
            if (gasStaff.getMerId().equals(merchant.getId())) {
                gasOrder.setGasStaffId(gasStaff.getId());
                gasOrder.setGasStaffName(gasStaff.getName());
            }
        }

        // 油站是否开启班组
        BsGasClassGroupTask groupTask = gasClassGroupTaskService.getCurrentTaskByMerId(merchant.getId());
        if (groupTask != null) {
            gasOrder.setGasClassGroupId(groupTask.getGasClassGroupId());
            gasOrder.setGasClassGroupName(groupTask.getGasClassGroupName());
            gasOrder.setGasClassGroupTaskId(groupTask.getId());
        }

        gasOrderService.addGasOrder(gasOrder);

        orderChild.setMerNo(merchant.getMerNo());
        orderChild.setProductName(merchant.getMerName());
        orderChild.setProductImg(merchant.getMerLogo());
        orderChild.setProductSpecName("默认");
        orderChild.setProductCount(1);
        orderChild.setProductActualPrice(gasOrder.getPayablePrice());
        return orderChild;
    }

}
